@@ -1,19 +1,21 @@
-from graphql.backend.core import GraphQLCoreBackend
-from graphql.execution import execute, ExecutionResult
+import re
+from dataclasses import dataclass, fields, replace
+from functools import wraps
+from typing import List, Union
+
+from graphql.error import GraphQLError
+from graphql.execution import ExecutionResult as GraphqlExecutionResult
 
 try:
-    from graphql.validation import (
-        validate,
-        specified_rules,
-        ValidationContext,
-        ValidationRule,
-    )
-
     from graphql.language import (
         DefinitionNode,
         FragmentSpreadNode,
         OperationDefinitionNode,
+        parse,
     )
+    from graphql.validation import ValidationContext, ValidationRule, validate
+
+    ExecutionResult = GraphqlExecutionResult
 
     def get_schema(ctx: ValidationContext):
         return ctx.schema
@@ -21,16 +23,28 @@ try:
     def get_ast(ctx: ValidationContext):
         return ctx.document
 
+    def get_optype(schema, definition):
+        operation_type = definition.operation.name.title()
+        return schema.get_type(operation_type)
 
 except ImportError:
-    from graphql.validation.validation import ValidationContext
-    from graphql.validation.rules.base import ValidationRule
-
     from graphql.language.ast import (
         Definition as DefinitionNode,
         FragmentSpread as FragmentSpreadNode,
         OperationDefinition as OperationDefinitionNode,
     )
+    from graphql.language.parser import parse
+    from graphql.validation.rules.base import ValidationRule
+    from graphql.validation.validation import ValidationContext, validate
+
+    class ExecutionResult(GraphqlExecutionResult):
+        def __init__(self, data=None, errors=None, extensions=None):
+            super().__init__(
+                data=data,
+                errors=errors,
+                extensions=extensions,
+                invalid=bool(errors),
+            )
 
     def get_schema(ctx: ValidationContext):
         return ctx.get_schema()
@@ -38,15 +52,9 @@ except ImportError:
     def get_ast(ctx: ValidationContext):
         return ctx.get_ast()
 
-
-import re
-from functools import partial
-
-from dataclasses import dataclass, fields, replace
-
-from typing import Any, Union, List
-from graphql.language.ast import Document
-from graphql.type.schema import GraphQLSchema
+    def get_optype(schema, definition):
+        operation_type = definition.operation.title()
+        return schema.get_type(operation_type)
 
 
 # Adapted from this response in Stackoverflow
@@ -86,7 +94,7 @@ _missing_limits = Limits()
 DEFAULT_LIMITS = Limits(depth=20, selections=None, complexity=100)
 
 
-class ResourceLimitReached(Exception):
+class ResourceLimitReached(GraphQLError):
     pass
 
 
@@ -124,9 +132,10 @@ def limits_for_field(field, old_limits):
 
 def check_resource_usage(
     schema,
-    selection_set,
+    node,
     validation_context,
     limits,
+    on_error,
     auto_snakecase=False,
     level=0,
 ):
@@ -137,8 +146,12 @@ def check_resource_usage(
         limits.depth is not MISSING
     ), "missing should be already resolved here"
     if limits.depth and max_depth > limits.depth:
-        raise DepthLimitReached("Query is too deep")
-    for field_orig in selection_set.selections:
+        on_error(
+            DepthLimitReached(
+                "Query is too deep",
+            )
+        )
+    for field_orig in node.selection_set.selections:
         fieldname = field_orig.name.value
         # ignore introspection queries
         if fieldname.startswith("__"):
@@ -154,9 +167,10 @@ def check_resource_usage(
             sub_limits = limits_for_field(schema_field, limits)
             new_depth, local_selections = check_resource_usage(
                 schema_field.type,
-                field.selection_set,
+                field,
                 validation_context,
                 sub_limits,
+                on_error=on_error,
                 auto_snakecase=auto_snakecase,
                 level=level + 1,
             )
@@ -166,7 +180,7 @@ def check_resource_usage(
                 and (new_depth - level) * local_selections
                 > sub_limits.complexity
             ):
-                raise ComplexityLimitReached("Query is too complex")
+                on_error(ComplexityLimitReached("Query is too complex", node))
             # ignore selection_set fields because we have depth for that
             selections += local_selections
             if new_depth > max_depth:
@@ -175,76 +189,101 @@ def check_resource_usage(
             selections += 1
 
         if limits.selections and selections > limits.selections:
-            raise SelectionsLimitReached("Query selects too much")
+            on_error(SelectionsLimitReached("Query selects too much", node))
     return max_depth, selections
 
 
 class LimitsValidationRule(ValidationRule):
-    def __init__(
-        self,
-        validation_context: ValidationContext,
-        default_limits=None,
-    ):
-        schema = get_schema(validation_context)
-        document: List[DefinitionNode] = get_ast(validation_context)
+    default_limits = None
+    # TODO: find out if auto_camelcase is set
+    # But no priority as this code works also
+    auto_snakecase = True
+
+    def __init__(self, context):
+        super().__init__(context)
+        schema = get_schema(self.context)
+        # if not set use schema to get defaults or set in case no limits
+        # are found to DEFAULT:LIMITS
+        if not self.default_limits:
+            self.default_limits = getattr(
+                schema, "get_default_limits", lambda: DEFAULT_LIMITS
+            )()
+
+    def enter(self, node, key, parent, path, ancestors):
+        if parent is not None:
+            return None
+        schema = get_schema(self.context)
+
+        document: List[DefinitionNode] = get_ast(self.context)
         for definition in document.definitions:
             if not isinstance(definition, OperationDefinitionNode):
                 continue
-            operation_type = definition.operation
-            maintype = getattr(schema, f"get_{operation_type}_type")()
+            maintype = get_optype(schema, definition)
             if hasattr(maintype, "graphene_type"):
                 maintype = maintype.graphene_type
+            #
             check_resource_usage(
                 maintype,
-                definition.selection_set,
-                validation_context,
-                default_limits,
-                auto_snakecase=getattr(schema, "auto_camelcase", False),
+                definition,
+                self.context,
+                self.default_limits,
+                self.report_error,
+                auto_snakecase=self.auto_snakecase,
             )
 
+    if not hasattr(ValidationRule, "report_error"):
 
-def execute_and_validate(
-    schema,  # type: GraphQLSchema
-    document_ast,  # type: Document
-    *args,  # type: Any
-    default_limits=None,
-    **kwargs,  # type: Any
-):
-    do_validation = kwargs.get("validate", True)
-    if do_validation:
-        if callable(default_limits):
-            default_limits = default_limits()
-        validation_errors = validate(
-            schema,
-            document_ast,
-            [
-                *specified_rules,
-                partial(LimitsValidationRule, default_limits=default_limits),
-            ],
-        )
-        if validation_errors:
-            return ExecutionResult(errors=validation_errors, invalid=True)
-
-    return execute(schema, document_ast, *args, **kwargs)
+        def report_error(self, error):
+            self.context.report_error(error)
 
 
-class ProtectorBackend(GraphQLCoreBackend):
+_rules = [LimitsValidationRule]
+
+
+def decorate_limits(fn):
+    @wraps(fn)
+    def wrapper(superself, query, *args, check_limits=True, **kwargs):
+        if check_limits:
+            try:
+                document_ast = parse(query)
+            except GraphQLError as error:
+                return ExecutionResult(data=None, errors=[error])
+
+            schema = getattr(superself, "graphql_schema", superself)
+            schema.get_default_limits = superself.get_default_limits
+
+            validation_errors = validate(
+                getattr(superself, "graphql_schema", superself),
+                document_ast,
+                _rules,
+            )
+            if validation_errors:
+                return ExecutionResult(errors=validation_errors)
+        return fn(superself, query, *args, **kwargs)
+
+    return wrapper
+
+
+def decorate_limits_async(fn):
+    decorated = decorate_limits(fn)
+
+    @wraps(fn)
+    async def wrapper(superself, *args, **kwargs):
+        return await decorated(superself, *args, **kwargs)
+
+    return wrapper
+
+
+class SchemaMixin:
     default_limits = None
 
-    def __init__(self, *args, limits=Limits(), **kwargs):
-        super().__init__(*args, **kwargs)
-        self.default_limits = limits
-
-    def document_from_string(self, schema, document_string):
-        document = super().document_from_string(schema, document_string)
-        document.execute = partial(
-            execute_and_validate,
-            schema,
-            document.document_ast,
-            default_limits=self.get_default_limits,
-            **self.execute_params,
-        )
-        return document
+    def __init_subclass__(cls, **kwargs):
+        if hasattr(cls, "execute"):
+            cls.execute = decorate_limits(cls.execute)
+        if hasattr(cls, "execute_async"):
+            cls.execute_async = decorate_limits_async(cls.execute_async)
+        if hasattr(cls, "subscribe"):
+            cls.subscribe = decorate_limits_async(cls.subscribe)
 
     def get_default_limits(self):
         return merge_limits(
